@@ -1,7 +1,7 @@
 /* embed.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: embed.c,v 1.33 2002/07/18 03:42:43 josh Exp $
+ *     $Id: embed.c,v 1.40 2002/08/26 14:15:45 grunblatt Exp $
  *  Overview:
  *     The Parrot embedding interface.
  *  Data Structure and Algorithms:
@@ -35,7 +35,7 @@ Parrot_new(void)
 }
 
 void
-Parrot_init(struct Parrot_Interp *interpreter)
+Parrot_init(struct Parrot_Interp *interpreter, void* stacktop)
 {
     /* This function currently unused, but is here in case we need it later. */
 
@@ -44,6 +44,8 @@ Parrot_init(struct Parrot_Interp *interpreter)
         world_inited = 1;
         init_world();
     }
+
+    if (stacktop) interpreter->lo_var_ptr = stacktop;
 }
 
 void
@@ -152,7 +154,8 @@ Parrot_readbc(struct Parrot_Interp *interpreter, char *filename)
 
         cursor = (char *)program_code;
 
-        while ((read_result = PIO_read(interpreter, io, cursor, chunk_size)) > 0) {
+        while ((read_result = 
+                PIO_read(interpreter, io, cursor, chunk_size)) > 0) {
             program_size += read_result;
             chunk_size = 1024;
             program_code =
@@ -197,7 +200,8 @@ Parrot_readbc(struct Parrot_Interp *interpreter, char *filename)
 
 #else   /* HAS_HEADER_SYSMMAN */
 
-        fprintf(stderr, "Parrot VM: uncaught error occurred reading file or mmap not available.\n");
+        fprintf(stderr,
+                "Parrot VM: uncaught error occurred reading file or mmap not available.\n");
         return NULL;
 
 #endif  /* HAS_HEADER_SYSMMAN */
@@ -232,12 +236,39 @@ Parrot_loadbc(struct Parrot_Interp *interpreter, struct PackFile *pf)
     interpreter->code = pf;
 }
 
+void
+Parrot_setup_argv(struct Parrot_Interp *interpreter, int argc, char ** argv)
+{
+    int i;
+    PMC *userargv;
+
+    if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
+        fprintf(stderr,
+                "*** Parrot VM: Setting up ARGV array in P0.  Current argc: %d ***\n",
+                argc);
+    }
+    
+    userargv = pmc_new(interpreter, enum_class_PerlArray);
+    /* immediately anchor pmc to root set */
+    interpreter->ctx.pmc_reg.registers[0] = userargv;
+
+    for (i = 0; i < argc; i++) {
+        /* Run through argv, adding everything to @ARGS. */
+        STRING *arg = string_make(interpreter, argv[i], strlen(argv[i]),
+                                  0, BUFFER_external_FLAG, 0);
+
+        if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
+            fprintf(stderr, "\t%d: %s\n", i, argv[i]);
+        }
+
+        userargv->vtable->push_string(interpreter, userargv, arg);
+    }
+}
+
 void
 Parrot_runcode(struct Parrot_Interp *interpreter, int argc, char *argv[])
 {
     INTVAL i;
-    PMC *userargv;
-    KEY key;
 
     /* Debugging mode nonsense. */
     if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
@@ -269,34 +300,10 @@ Parrot_runcode(struct Parrot_Interp *interpreter, int argc, char *argv[])
     
 #endif
 
-    if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
-        fprintf(stderr,
-                "*** Parrot VM: Setting up ARGV array in P0.  Current argc: %d ***\n",
-                argc);
-    }
-    
     /* Set up @ARGS (or whatever this language calls it).
        XXX Should this be Array or PerlArray?             */
-    
-    userargv = pmc_new(interpreter, enum_class_PerlArray);
-    /* immediately anchor pmc to root set */
-    interpreter->ctx.pmc_reg.registers[0] = userargv;
 
-    key.atom.type = enum_key_int;
-    key.next = NULL;
-
-    for (i = 0; i < argc; i++) {
-        /* Run through argv, adding everything to @ARGS. */
-        STRING *arg = string_make(interpreter, argv[i], strlen(argv[i]),
-                                  0, BUFFER_external_FLAG, 0);
-
-        if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
-            fprintf(stderr, "\t" INTVAL_FMT ": %s\n", i, argv[i]);
-        }
-
-        key.atom.val.int_val = i;
-        userargv->vtable->set_string_keyed(interpreter, userargv, &key, arg);
-    }
+    Parrot_setup_argv(interpreter, argc, argv);
     
     /* Let's kick the tires and light the fires--call interpreter.c:runops. */
     runops(interpreter, interpreter->code, 0);
@@ -326,7 +333,8 @@ Parrot_runcode(struct Parrot_Interp *interpreter, int argc, char *argv[])
                        interpreter->op_info_table[j].full_name,
                        interpreter->profile[j].numcalls,
                        interpreter->profile[j].time,
-                       interpreter->profile[j].time / (FLOATVAL)interpreter->profile[j].numcalls
+                       interpreter->profile[j].time / 
+                           (FLOATVAL)interpreter->profile[j].numcalls
                 );
             }
         }
@@ -342,7 +350,8 @@ Parrot_runcode(struct Parrot_Interp *interpreter, int argc, char *argv[])
     }
 
     if (Interp_flags_TEST(interpreter, PARROT_DEBUG_FLAG)) {
-        /* Give the souls brave enough to activate debugging an earful about GC. */
+        /* Give the souls brave enough to activate debugging an earful 
+         * about GC. */
         fprintf(stderr, "\
 *** Parrot VM: Dumping GC info ***\n\
 \tTotal memory allocated: %u\n\
@@ -376,8 +385,32 @@ Parrot_destroy(struct Parrot_Interp *interp)
     free(interp);
 }
 
+/* XXX Doesn't handle arguments with spaces */
+static char*
+argv_join(char ** argv)
+{
+    char* command;
+    char* p;
+    int space = 0;
+    int i;
+
+    for (i = 0; argv[i]; i++)
+        space += strlen(argv[i]) + 1;
+
+    command = (char*) malloc(space == 0 ? 1 : space);
+    p = command;
+    for (i = 0; argv[i]; i++) {
+        strcpy(p, argv[i]);
+        p += strlen(argv[i]);
+        *(p++) = ' ';
+    }
+    if (p > command) p--;
+    *p = '\0';
+    return command;
+}
+
 void
-Parrot_debug(struct Parrot_Interp *interpreter)
+Parrot_debug(struct Parrot_Interp *interpreter, int argc, char ** argv)
 {
     PDB_t *pdb;
     const char *command;
@@ -387,7 +420,8 @@ Parrot_debug(struct Parrot_Interp *interpreter)
     interpreter->pdb = pdb;
     pdb->cur_opcode = interpreter->code->byte_code;
 
-    PDB_init(interpreter,NULL);
+    /* Parrot_setup_argv(interpreter, argc, argv); */
+    PDB_init(interpreter, argv_join(argv));
     PDB_disassemble(interpreter,NULL);
     while (!(pdb->state & PDB_EXIT)) {
         PDB_get_command(interpreter);
@@ -401,7 +435,7 @@ Parrot_disassemble(struct Parrot_Interp *interpreter)
 {
     PDB_t *pdb;
     PDB_line_t *line;
-    char *dis, *c;
+    char *c;
 
     pdb = (PDB_t *)mem_sys_allocate(sizeof(PDB_t));
 
