@@ -1,7 +1,7 @@
 /* global_setup.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: global_setup.h,v 1.8 2002/07/04 18:31:20 mrjoltcola Exp $
+ *     $Id: global_setup.h,v 1.12 2002/08/19 23:15:52 tom Exp $
  *  Overview:
  *      Contains declarations of global data and the functions
  *      that initialize that data.
@@ -22,14 +22,17 @@ void Parrot_PerlInt_class_init(INTVAL);
 void Parrot_PerlNum_class_init(INTVAL);
 void Parrot_PerlString_class_init(INTVAL);
 void Parrot_Array_class_init(INTVAL);
+void Parrot_Boolean_class_init(INTVAL);
 void Parrot_PerlArray_class_init(INTVAL);
 void Parrot_PerlHash_class_init(INTVAL);
 void Parrot_Pointer_class_init(INTVAL);
 void Parrot_IntQueue_class_init(INTVAL);
 void Parrot_Sub_class_init(INTVAL);
 void Parrot_Coroutine_class_init(INTVAL);
-void Parrot_Closure_class_init(INTVAL);
 void Parrot_Continuation_class_init(INTVAL);
+void Parrot_CSub_class_init(INTVAL);
+void Parrot_MultiArray_class_init(INTVAL);
+void Parrot_Key_class_init(INTVAL);
 
 void init_world(void);
 
