@@ -1,6 +1,6 @@
 /* packfile.h
 *
-* $Id: packfile.h,v 1.28 2002/07/18 04:12:20 josh Exp $
+* $Id: packfile.h,v 1.31 2002/08/19 23:15:52 tom Exp $
 *
 * History:
 *  Rework by Melvin; new bytecode format, make bytecode portable.
@@ -23,6 +23,7 @@
 
 /*
 ** Bytes that we don't have to reorder
+*  PACKFILE_HEADER_BYTES must be an integer times sizeof(opcode_t).
 */
 #define PACKFILE_HEADER_BYTES 16
 
@@ -34,7 +35,7 @@ struct PackFile_Header {
     unsigned char flags;
     unsigned char floattype;
     unsigned char pad[10];
-    /* Start words/opcodes on 8-byte boundary */
+    /* Start words/opcodes on 16-byte boundary */
     opcode_t magic;
     opcode_t opcodetype; 
     opcode_t fixup_ss;  
@@ -49,12 +50,14 @@ struct PackFile_FixupTable {
 #define PFC_NONE    '\0'
 #define PFC_NUMBER  'n'
 #define PFC_STRING  's'
+#define PFC_KEY     'k'
 
 struct PackFile_Constant {
     opcode_t type;
     opcode_t integer;
     FLOATVAL number;
     STRING *string;
+    PMC *key;
 };
 
 
@@ -100,11 +103,13 @@ opcode_t PackFile_unpack(struct Parrot_Interp *interpreter,
 
 void PackFile_FixupTable_clear(struct PackFile_FixupTable * self);
 
-INTVAL PackFile_FixupTable_unpack(struct PackFile_FixupTable * self, opcode_t * packed, opcode_t packed_size);
+INTVAL PackFile_FixupTable_unpack(struct PackFile_FixupTable * self, 
+                                  opcode_t * packed, opcode_t packed_size);
 
 opcode_t PackFile_FixupTable_pack_size(struct PackFile_FixupTable * self);
 
-void PackFile_FixupTable_pack(struct PackFile_FixupTable * self, opcode_t * packed);
+void PackFile_FixupTable_pack(struct PackFile_FixupTable * self, 
+                              opcode_t * packed);
 
 /*
 ** PackFile_ConstTable Functions:
@@ -131,24 +136,31 @@ struct PackFile_Constant *PackFile_Constant_new(void);
 
 opcode_t PackFile_Constant_pack_size(struct PackFile_Constant * self);
 
-void PackFile_Constant_pack(struct PackFile_Constant * self, opcode_t * packed);
+void PackFile_Constant_pack(struct PackFile_Constant * self, 
+                            opcode_t * packed);
 
 void PackFile_Constant_destroy(struct PackFile_Constant * self);
 
 INTVAL PackFile_Constant_unpack(struct Parrot_Interp *interpreter,
-                                 struct PackFile *packfile,
-                                 struct PackFile_Constant *self,
-                                 opcode_t *packed, opcode_t packed_size);
+                                struct PackFile *packfile,
+                                struct PackFile_Constant *self,
+                                opcode_t *packed, opcode_t packed_size);
 
 INTVAL PackFile_Constant_unpack_number(struct PackFile * pf,
-                                        struct PackFile_Constant * self,
-                                        opcode_t * packed, opcode_t packed_size);
+                                       struct PackFile_Constant * self,
+                                       opcode_t * packed, opcode_t packed_size);
 
 INTVAL PackFile_Constant_unpack_string(struct Parrot_Interp *interpreter,
-                                        struct PackFile * pf,
-                                        struct PackFile_Constant *self,
-                                        opcode_t *packed,
-                                        opcode_t packed_size);
+                                       struct PackFile * pf,
+                                       struct PackFile_Constant *self,
+                                       opcode_t *packed,
+                                       opcode_t packed_size);
+
+INTVAL PackFile_Constant_unpack_key(struct Parrot_Interp *interpreter,
+                                    struct PackFile * pf,
+                                    struct PackFile_Constant *self,
+                                    opcode_t *packed,
+                                    opcode_t packed_size);
 
 opcode_t PackFile_fetch_op(struct PackFile *pf, opcode_t *stream);
 
