@@ -1,7 +1,7 @@
 /* register.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: resources.h,v 1.35 2002/07/18 04:30:42 mongo Exp $
+ *     $Id: resources.h,v 1.38 2002/08/21 08:02:25 mongo Exp $
  *  Overview:
  *     Defines the resource allocation API
  *  Data Structure and Algorithms:
@@ -29,7 +29,9 @@ struct Memory_Pool {
     void (*compact)(struct Parrot_Interp *, struct Memory_Pool *);
     size_t minimum_block_size;
     size_t total_allocated; /* total bytes allocated to this pool */
-    size_t reclaimable;     /* bytes that can be reclaimed (approximate) */
+    size_t guaranteed_reclaimable;     /* bytes that can definitely be reclaimed*/
+    size_t possibly_reclaimable;     /* bytes that can possibly be reclaimed 
+                                      * (above plus COW-freed bytes) */
     FLOATVAL reclaim_factor; /* minimum percentage we will reclaim */
 };
 
@@ -38,8 +40,10 @@ struct Memory_Pool {
 
 void *Parrot_allocate(struct Parrot_Interp *, void *, size_t size);
 void *Parrot_allocate_string(struct Parrot_Interp *, STRING *, size_t size);
-void *Parrot_reallocate(struct Parrot_Interp *interpreter, void *from, size_t tosize);
-void *Parrot_reallocate_string(struct Parrot_Interp *interpreter, STRING *, size_t tosize);
+void *Parrot_reallocate(struct Parrot_Interp *interpreter, 
+                        void *from, size_t tosize);
+void *Parrot_reallocate_string(struct Parrot_Interp *interpreter, 
+                               STRING *, size_t tosize);
 
 void Parrot_initialize_memory_pools(struct Parrot_Interp *);
 
@@ -69,7 +73,7 @@ struct Stash {
 #define ACTIVE_BUFFERS 5
 #define TOTAL_PMCS 6
 #define TOTAL_BUFFERS 7
-#define HEADERS_ALLOC_SINCE_COLLECT 8
+#define HEADER_ALLOCS_SINCE_COLLECT 8
 #define MEM_ALLOCS_SINCE_COLLECT 9
 #define TOTAL_COPIED 10
 
