@@ -1,7 +1,7 @@
 /* string.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: string.h,v 1.43 2002/07/18 04:30:42 mongo Exp $
+ *     $Id: string.h,v 1.46 2002/08/23 07:55:22 mongo Exp $
  *  Overview:
  *     This is the api header for the string subsystem
  *  Data Structure and Algorithms:
@@ -25,7 +25,11 @@ struct parrot_string_t {
     void *bufstart;
     UINTVAL buflen;
     UINTVAL flags;
+#if GC_DEBUG
+    UINTVAL version;
+#endif
     UINTVAL bufused;
+    void *strstart;
     UINTVAL strlen;
     const ENCODING *encoding;
     const CHARTYPE *type;
@@ -41,10 +45,22 @@ typedef struct {
     void *bufstart;
     UINTVAL buflen;
     UINTVAL flags;
+#if GC_DEBUG
+    UINTVAL version;
+#endif
 } Buffer;
 
 typedef struct parrot_string_t String;
 
+
+/* Tail added to end of string buffers; used for COW GC */
+struct Buffer_Tail {
+    unsigned char flags;
+};
+typedef enum TAIL_flag {
+    TAIL_moved_FLAG = 1 << 0
+} TAIL_flags;
+
 /* Buffer flags */
 typedef enum BUFFER_flag {
     /* bits the GC can keep its dirty mitts off of */
@@ -76,12 +92,18 @@ typedef enum BUFFER_flag {
     /* For debugging, report when this buffer gets moved around */
     BUFFER_report_FLAG = 1 << 16,
     /* Generation in the GC pools */
-    BUFFER_generation_FLAG = 1 << 17 | 1 << 18
+    BUFFER_generation_FLAG = 1 << 17 | 1 << 18,
+    /* Buffer header has a strstart which needs to be updated with bufstart */
+    BUFFER_strstart_FLAG = 1 << 19,
+    /* Buffer's memory data is in this header's header pool's memory pool */
+    /* for now, this is true in constant headers for constant buffer data,and 
+     * true for non-constant headers pointing at non-constant buffer data */
+    BUFFER_selfpoolptr_FLAG = 1 << 20
 } BUFFER_flags;
 
 /* stringinfo parameters */
 #define STRINGINFO_HEADER   1
-#define STRINGINFO_BUFSTART 2
+#define STRINGINFO_STRSTART 2
 #define STRINGINFO_BUFLEN   3
 #define STRINGINFO_FLAGS    4
 #define STRINGINFO_BUFUSED  5
