@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 #
-# $Id: basic.pl,v 1.13 2002/06/22 19:21:22 clintp Exp $
+# $Id: basic.pl,v 1.14 2002/07/31 15:45:17 clintp Exp $
 # $Log: basic.pl,v 
 # Revision 1.11  2002/06/16 21:23:28  clint
 # Floating Point BASI
@@ -65,6 +65,8 @@ MAINLOOPNR:
 	eq I1, 1, MAINLOOPR
 	save S0
 	bsr STRIPSPACE
+	restore S0	# Mystery bug.
+	save S0		# CAP 7/31/02
 	bsr TOKENIZER
 	bsr REVERSESTACK
 	restore I5
