@@ -27,10 +27,12 @@ and unary operators, and for "magic" things like guards and C<loop>.
 
 =cut
 
+use Data::Dumper;
+use Carp 'confess';
+use strict;
 use P6C::Context;
 use P6C::Nodes;
 use P6C::Util qw(:all);
-use strict;
 
 BEGIN { # Add types for builtin binary operators.
     # types for symmetric operators:
@@ -48,7 +50,7 @@ BEGIN { # Add types for builtin binary operators.
 	 # NOTE: Actually, according to apo 3, boolean operators
 	 # propagate values in their surrounding context (even though
 	 # they may evaluate in boolean context?).  So we can't quite
-	 # do this.
+	 # do this:
 #	 bool => [ qw(&& ~~ ||) ],
 	);
 
@@ -88,22 +90,35 @@ sub P6C::Binop::ctx_right {
     my ($x, $ctx) = @_;
     my $op = $x->op;
     if (ref($op) && $op->isa('P6C::hype')) {
-	# XXX: do we need to propagate hyper context?  This might be
-	# useful for coalescing sequences of hyper-ops.  For now, just
-	# ignore it.
+	# XXX: is_array_expr is a hack, so this may cause some
+	# problems.  However, it works in straightforward cases.
 	my $newctx = new P6C::Context;
-	$x->l->ctx_right($newctx);
-	$x->r->ctx_right($newctx);
+	for ($x->l, $x->r) {
+	    if (is_array_expr($_)) {
+		$newctx->type('PerlArray');
+	    } else {
+		$newctx->type('PerlUndef');
+	    }
+	    $_->ctx_right($newctx);
+	}
 
     } elsif ($op eq '=') {
 	# Special-case assignment operator to enforce context l -> r
-	# XXX: not sure what to do with +=, etc.
-
 	# Propagate context:
 	$x->l->ctx_left($x->r, $ctx);
 	# give incoming context to left side
 	$x->l->ctx_right($ctx);
 
+    } elsif ($op =~ /^([^=]+)=$/) {
+	# Turn this into a normal, non-inplace operator and try again.
+	# Yuck.
+	my $ltmp = deep_copy($x->l);
+	my $new_rhs = new P6C::Binop op => $1, l => $ltmp, r => $x->r;
+	$x->op('=');
+	$x->r($new_rhs);
+	$x->ctx_right($ctx);
+	return;
+
     } elsif ($op eq ',') {
 	# List of items.
 	# XXX: this is gross.
@@ -115,18 +130,17 @@ sub P6C::Binop::ctx_right {
 	# This operator determines context for its operands.
 	my ($ltype, $rtype) = @{$P6C::Context::CONTEXT{"infix $op"}->type};
 	my $opctx = $ctx->copy;
+	$opctx->flatten(0);	# XXX:
 	$opctx->type($ltype);
 	$x->l->ctx_right($opctx);
 	$opctx->type($rtype);
 	$x->r->ctx_right($opctx);
 
     } else {
-	# We know nothing about the context.  Say so, and propagate
-	# the surrounding context.
-	diag "No context information for `infix $op'";
-	my $opctx = new P6C::Context;
-	$x->l->ctx_right($opctx);
-	$x->r->ctx_right($opctx);
+	# We know nothing about the context, so propagate
+	# the surrounding context (works for logical ops).
+	$x->l->ctx_right($ctx);
+	$x->r->ctx_right($ctx);
     }
 
     # Store away our context.
@@ -161,43 +175,51 @@ sub P6C::variable::ctx_left {
 }
 
 ##############################
+# Arrays are always integer-indexed.  Hashes are currently
+# string-indexed.  If that changes to PMC's change this.
+my %indextype = (PerlHash	=> 'str',
+		 PerlArray	=> 'int');
+
 sub P6C::indices::ctx_right {
     my ($x, $ctx) = @_;
-
+    if ($x->{ctx}) {
+	# Context gets called twice for assignment to a subscript.
+	# Don't propagate to the indices the second time, or they'll
+	# end up in void context, which is bad.
+	return;
+    }
 # Indices take their context's arity from the outer context, but its
 # type from the index type.  For example, in C<(str $a, num $b) =
 # @x[1,2]>, the outer context is C<[str, int]>, the index type is
 # C<PerlArray>, and the resulting indexing context is C<[int, int]>.
 
-# Arrays are always integer-indexed.  Hashes are currently
-# string-indexed.  If that changes to PMC's change this.
-    my %indextype = qw(PerlHash  str
-		       PerlArray int);
-    my $index_ctx;
-    if ($ctx->is_tuple) {
+    if (defined($x->indices)) {
+	my $index_ctx;
+	if ($x->type eq 'Sub') {
+ 	    $index_ctx = $P6C::Context::DEFAULT_ARGUMENT_CONTEXT;
 
-	$index_ctx = new P6C::Context
-	    type => [($indextype{$x->type}) x @{$ctx->type}];
+	} elsif ($ctx->is_tuple) {
+	    $index_ctx = new P6C::Context
+		type => [($indextype{$x->type}) x @{$ctx->type}];
 
-    } elsif ($ctx->is_scalar || $ctx->type eq 'void') {
-	# XXX: this causes problems with things like C<$x = @a[@i]>.
-	# I don't know what that should do.
-	$index_ctx = new P6C::Context type => [$indextype{$x->type}];
+	} elsif ($ctx->is_scalar || $ctx->type eq 'void') {
+	    # XXX: this causes problems with things like C<$x = @a[@i]>.
+	    # I don't know what that should do.
+	    $index_ctx = new P6C::Context type => $indextype{$x->type};
 
-    } elsif ($ctx->is_array) {
-	$index_ctx = new P6C::Context type => 'PerlArray';
+	} elsif ($ctx->is_array) {
+	    $index_ctx = new P6C::Context type => 'PerlArray';
 
-    } else {
-	use Data::Dumper;
-	use Carp 'confess';
-	confess "index contest: ", Dumper($ctx);
+	} else {
+	    confess "index contest: ", Dumper($ctx);
+	}
+
+	$x->indices->ctx_right($index_ctx);
     }
 
-    $x->indices->ctx_right($index_ctx);
-
     # unused for now.  This could be used to fix the scalar indexing
     # problem above.
-   $x->{ctx} = $ctx->copy;
+    $x->{ctx} = $ctx->copy;
 }
 
 sub P6C::indices::ctx_left {
@@ -207,27 +229,26 @@ sub P6C::indices::ctx_left {
 
     # Now figure out context for RHS.
     my $rctx = new P6C::Context;
+    my $indexctx;
 
-    if ($x->indices->isa('P6C::sv_literal')
-	|| ($x->indices->isa('P6C::variable')
-	    && is_scalar($x->indices->type))) {
-	# Scalar expression.  Note that this misses a lot of things.
-	$rctx->type('PerlUndef');
-	$x->indices->ctx_right(new P6C::Context type => $x->type);
-
-    } elsif ($x->indices->isa('P6C::Binop') && $x->op eq ',') {
+    if ($x->indices->isa('P6C::Binop') && $x->indices->op eq ',') {
 	# tuple => propagate arity to other side.
+	# XXX: doesn't flatten!
 	my @things = $x->indices->flatten_leftop(',');
 	$rctx->type([('PerlUndef') x @things]);
-	$x->indices->ctx_right(new P6C::Context
-				   type => [($x->type) x @things]);
+	$indexctx = new P6C::Context type => [($indextype{$x->type}) x @things];
+
+    } elsif (is_array_expr($x->indices)) {
+	$rctx->type('PerlArray');
+	$indexctx = new P6C::Context type => 'PerlArray';
 
     } else {
-	# Indexing by something else => fallback to array.
-	$rctx->type('PerlArray');
-	$x->indices->ctx_right(new P6C::Context type => 'PerlArray');
+	$rctx->type('PerlUndef');
+	$indexctx = new P6C::Context type => $indextype{$x->type};
     }
-    
+    $x->indices->ctx_right($indexctx);
+
+    $x->{ctx} = $indexctx;
     return $rctx;
 }
 
@@ -241,13 +262,26 @@ sub P6C::subscript_exp::ctx_right {
 	unimp "multi-level subscripting";
     }
     my $index = $x->subscripts(0);
-    # The base variable is indexed as a $index->type, so give it that
-    # context.
-    $x->thing->ctx_right(new P6C::Context type => $index->type);
+    if ($index->type eq 'Sub') {
+	# Closure.
+	my $ictx;
+	if ($x->thing->can('name')) {
+	    $ictx = arg_context($x->thing->name);
+	} else {
+	    $ictx = $P6C::Context::DEFAULT_ARGUMENT_CONTEXT;
+	}
+	$x->thing->ctx_right(new P6C::Context type => 'PerlUndef');
+	$index->ctx_right($ictx);
 
-    # Propagate context to indices as well.  For the moment, they just
-    # use the index arity.
-    $index->ctx_right($ctx);
+    } else {
+	# The base variable is indexed as a $index->type, so give it that
+	# context.
+	$x->thing->ctx_right(new P6C::Context type => $index->type);
+	
+	# Propagate context to indices as well.  For the moment, they just
+	# use the index arity.
+	$index->ctx_right($ctx);
+    }
 
     $x->{ctx} = $ctx->copy;
 }
@@ -302,7 +336,7 @@ sub P6C::incr::ctx_right {
 	if ($subcontext) {
 	    $x->thing->ctx_right($subcontext);
 	} else {
-	    diag "No context for operator `$name'";
+#	    diag "No context for operator `$name'";
 	}
     }
     $x->{ctx} = $ctx->copy;
@@ -311,6 +345,8 @@ sub P6C::incr::ctx_right {
 ##############################
 sub ifunless_context {
     my ($x, $ctx) = @_;
+    $x->{ctx} = $ctx->copy;
+    $x->{ctx}{noreturn} = 1;
     my $boolctx = new P6C::Context type => 'bool';
     foreach (@{$x->args}) {
 	my ($sense, $test, $block) = @$_;
@@ -318,12 +354,11 @@ sub ifunless_context {
 	if (ref $test) {
 	    $test->ctx_right($boolctx);
 	}
-	$block->ctx_right($ctx);
+	$block->ctx_right($x->{ctx});
     }
 }
 
 sub for_context {
-    use P6C::Util 'flatten_leftop';
     my ($x, $ctx) = @_;
     my ($ary, $body) = @{$x->args->vals};
     my @streams = flatten_leftop($ary, ';');
@@ -355,20 +390,53 @@ END
     }
     $x->args->vals(0, [@stream_result]);
     # Get the body:
+    $ctx->{noreturn} = 1;
     $body->ctx_right($ctx);
+    delete $ctx->{noreturn};
+}
+
+sub foreach_context {
+    my ($x, $ctx) = @_;
+    my $args = $x->args->vals;
+    my $voidctx = new P6C::Context type => 'void';
+    if ($args->[0]) {
+	$args->[0]->ctx_right($voidctx);
+    }
+    $args->[1]->ctx_right(new P6C::Context type => 'PerlArray');
+    for (@{$args->[2]}) {
+	$_->ctx_right($voidctx);
+    }
 }
 
 BEGIN {
-    $P6C::Context::CONTEXT{'-'} = new P6C::Context type => ['PerlUndef'];
+    $P6C::Context::CONTEXT{'-'} = new P6C::Context type => 'PerlUndef';
     $P6C::Context::CONTEXT{if}
 	= $P6C::Context::CONTEXT{unless} =  \&ifunless_context;
     $P6C::Context::CONTEXT{'for'} = \&for_context;
+    $P6C::Context::CONTEXT{foreach} = \&foreach_context;
+    $P6C::Context::CONTEXT{when}
+	= $P6C::Context::CONTEXT{given}
+	= new P6C::Context type => [undef, 'void'];
+
     $P6C::Context::CONTEXT{while}
-	= $P6C::Context::CONTEXT{until} # = \&while_context;
-	   = new P6C::Context type => ['bool', 'void'];
+	= $P6C::Context::CONTEXT{until}
+	= new P6C::Context type => ['bool', 'void'];
+
+    $P6C::Context::CONTEXT{return} = new P6C::Context type => 'PerlArray';
     $P6C::Context::CONTEXT{print1} = new P6C::Context type => ['PerlUndef'];
-#     $P6C::Context::CONTEXT{for}
-# 	= new P6C::Context type => ['PerlArray', 'void'];
+    my $passthrough = sub {
+	my ($x, $ctx) = @_;
+	$ctx = $ctx->copy;
+	$ctx->{noreturn} = 1;
+	$x->args->ctx_right($ctx);
+    };
+    for (qw(try do CATCH BEGIN END INIT AUTOLOAD PRE POST NEXT LAST FIRST
+	    default)) {
+	$P6C::Context::CONTEXT{$_} = $passthrough;
+    }
+    for (qw(while when given)) {
+	$P6C::Context::CONTEXT{$_}{noreturn} = 1;
+    }
 }
 
 # Lookup context for a prefix operator.  If the sub hasn't been
@@ -379,22 +447,21 @@ sub arg_context {
     if (exists $P6C::Context::CONTEXT{$name}) {
 	return $P6C::Context::CONTEXT{$name};
     }
-    diag 'no context found for ', $name;
+#     diag "No context for $name";
     return $P6C::Context::DEFAULT_ARGUMENT_CONTEXT;
 }
 
 sub P6C::prefix::ctx_right {
     my ($x, $ctx) = @_;
     my $proto = arg_context($x->name, $ctx);
+    $x->{ctx} = $ctx->copy;
 
     if (ref($proto) eq 'CODE') {
 	# blech.
 	$proto->($x, $ctx);
-    } else {
-	$x->args->ctx_right($proto);
+    } elsif (ref $x->args) {
+	$x->args->ctx_right($proto->copy);
     }
-
-    $x->{ctx} = $ctx->copy;
 }
 
 ##############################
@@ -452,15 +519,7 @@ sub P6C::ternary::ctx_left {
 	if (!defined $other) {
 	    unimp "Assignment to ternary in too hairy a context.";
 	}
-	my $treecopy;
-	{
-	    use Data::Dumper;
-	    local $Data::Dumper::Purity = 1;
-	    local $Data::Dumper::Terse = 1;
-	    local $Data::Dumper::Deepcopy = 1;
-	    $treecopy = eval Dumper($other);
-	    die "Can't duplicate op-tree: $@" if $@;
-	}
+	my $treecopy = deep_copy $other;
 
 	$x->then->ctx_left($other, $ctx);
 	$x->{then_right} = $other;
@@ -478,23 +537,28 @@ sub P6C::ternary::ctx_left {
 }
 
 ##############################
-sub P6C::decl::ctx_right { }
+sub P6C::decl::ctx_right {
+    my ($x, $ctx) = @_;
+    unless ($ctx->type eq 'void') {
+	unimp "declaration in non-void context";
+    }
+    if (ref $x->vars eq 'ARRAY') {
+	$_->ctx_right($ctx) for @{$x->vars};
+    } else {
+	$x->vars->ctx_right($ctx);
+    }
+}
 
 sub P6C::decl::ctx_left {
-    # XXX: this may not be quite right.  If we're declaring a single
-    # item, we create a one-item context.  If more, a tuple context.
-    # Problem is, we may have thrown away the parens around the single
-    # item by this point.
     my ($x, $other, $ctx) = @_;
-    my @ctx = map { $_->type } @{$x->vars};
-    if (@ctx == 1) {
-	my $ctx = new P6C::Context type => $ctx[0];
-	if ($ctx[0] eq 'PerlArray') {
+    if (ref $x->vars ne 'ARRAY') { # single-variable declaration.
+ 	my $ctx = new P6C::Context type => $x->vars->type;
+	if ($ctx->is_array) {
 	    $ctx->flatten(1);
 	}
 	$other->ctx_right($ctx);
     } else {
-	# Tuple.
+	my @ctx = map { $_->type } @{$x->vars};
 	$other->ctx_right(new P6C::Context type => \@ctx);
     }
 }
@@ -519,24 +583,117 @@ sub P6C::sub_def::ctx_right {
 	$argctx = new P6C::Context type => [@types];
     }
     $P6C::Context::CONTEXT{$x->name} = $argctx;
-    $x->closure->ctx_right($ctx);
+    $x->{ctx} = $ctx->copy;
+    $x->{ctx}{is_sub_def} = 1;
+    delete $x->{ctx}{noreturn};
+    $x->closure->ctx_right($x->{ctx});
 }
 
 ##############################
+
+sub get_closure_params {
+    my ($x) = @_;
+    my @params;
+    if (defined($x->params)) {
+	if (UNIVERSAL::isa($x->params, 'P6C::params')) {
+	    @params = map { $_->var } @{$x->params->req}; # XXX:
+	} else {
+	    # Explicit parameter list in "-> $foo, $bar { ... }"
+	    foreach (flatten_leftop($x->params, ';')) {
+		push @params, flatten_leftop($_, ',');
+	    }
+	}
+    } else {
+	my %impl;
+	# Look for implicit param-vars like $^a:
+	map_preorder {
+	    if (UNIVERSAL::isa($_, 'P6C::variable')
+		&& $_->implicit) {
+		$impl{$_->name} = $_;
+	    }
+	} $x->block;
+	if (keys %impl) {
+	    @params = sort { $a->name cmp $b->name } values %impl;
+	}
+    }
+    return @params;
+}
+
+sub setup_catch_blocks {
+    my ($x) = @_;
+    my @catch;
+    map_preorder {
+	if (UNIVERSAL::isa($_, 'P6C::prefix')
+	    && $_->name eq 'CATCH') {
+	    push @catch, $_;
+	}
+    } $x->block;
+    if (@catch) {
+	$x->{catch} = [@catch];
+    }
+}
+
+sub is_anon_sub {
+    my ($x, $ctx) = @_;
+    return !($ctx->{noreturn}
+	     || $ctx->{is_sub_def}
+	     || ($ctx->{last_stmt} && $x->bare));
+}
+
+sub is_noreturn {
+    my ($x, $ctx) = @_;
+    return $ctx->{noreturn} || ($ctx->{last_stmt} && $x->bare);
+}
+
 sub P6C::closure::ctx_right {
     my ($x, $ctx) = @_;
-    if ($ctx->type ne 'void') {
-	unimp 'closure in non-void context';
+
+    $x->{ctx} = $ctx->copy;
+    $x->{ctx}{is_anon_sub} = is_anon_sub($x, $ctx);
+    $x->{ctx}{noreturn} = is_noreturn($x, $ctx);
+    delete $x->{ctx}{is_sub_def};
+
+    if ($x->{ctx}{is_anon_sub}) {
+	my @params = get_closure_params($x);
+	if (@params) {
+	    my $vals = new P6C::variable(name => '@_',
+					 type => 'PerlArray');
+	    my $paramvar;
+	    if (@params == 1 && is_array_expr($params[0]->type)) {
+		$paramvar = $params[0];
+	    } else {
+		$paramvar = \@params;
+	    }
+	    my $vars = new P6C::decl(vars => $paramvar,
+				     props => [],
+				     qual => new P6C::scope_class scope=>'my');
+	    my $init = new P6C::Binop(op => '=', l => $vars, r => $vals);
+	    if (defined $x->block) {
+		unshift @{$x->block}, $init;
+	    } else {
+		diag "Closure with no statements?";
+		$x->block([$init]);
+	    }
+	    $x->params(undef);	# will fill them in in IMCC.pm
+	}
     }
 
-    # NOTE: once we get return values in, we're in for serious pain
-    # here, since we have to evaluate the last statement in the
-    # _caller_'s context, which we don't know now.
-    if (defined $x->block) {	# real def.
-	my $voidctx = new P6C::Context type => 'void';
-	foreach my $stmt (@{$x->block}) {
-	    $stmt->ctx_right($voidctx);
+    if (!defined $x->block) {
+    } elsif (UNIVERSAL::isa($x->block, 'ARRAY')) {
+	# Sub block
+	# Look for CATCH blocks in the current block:
+	setup_catch_blocks($x);
+
+	if ($x->{ctx}{noreturn}) {
+	    P6C::Context::block_ctx($x->block, $x->{ctx});
+	} else {
+	    P6C::Context::block_ctx($x->block,
+				    new P6C::Context type => 'PerlArray');
 	}
+    } elsif ($x->block->isa('P6C::rule')) {
+	$x->block->ctx_right(new P6C::Context type => 'PerlUndef');
+    } else {
+	die "Internal error: closure body is ", $x->block;
     }
 }
 
@@ -544,10 +701,17 @@ sub P6C::closure::ctx_right {
 sub P6C::ValueList::ctx_right {
     my ($x, $ctx) = @_;
 
+    # XXX: hack for hyper-operators:
+    if (!defined($ctx->type)) {
+	$ctx->type('PerlArray');
+    }
+
     if ($ctx->is_tuple) {
 	my $min = @{$ctx->type} < @{$x->vals} ? @{$ctx->type} : @{$x->vals};
+	my $newctx = $ctx->copy;
 	for my $i (0 .. $min - 1) {
-	    $x->vals($i)->ctx_right(new P6C::Context type => $ctx->type->[$i]);
+	    $newctx->type($ctx->type->[$i]);
+	    $x->vals($i)->ctx_right($newctx);
 	}
 	my $voidctx = new P6C::Context type => 'void';
 	for my $i ($min .. $#{$x->vals}) {
@@ -555,7 +719,7 @@ sub P6C::ValueList::ctx_right {
 	}
 
     } elsif ($ctx->is_array) {
-	my $actx = new P6C::Context;
+	my $actx = $ctx->copy;
 	if ($ctx->flatten) {
 	    $actx->type('PerlArray');
 	} else {
@@ -566,14 +730,14 @@ sub P6C::ValueList::ctx_right {
 	}
 
     } elsif ($ctx->is_scalar || $ctx->type eq 'void') {
-	my $voidctx = new P6C::Context type => 'void';
+	my $voidctx = $ctx->copy;
+	$voidctx->type('void');
 	for my $i (0 .. $#{$x->vals} - 1) {
 	    $x->vals($i)->ctx_right($voidctx);
 	}
 	$x->vals->[-1]->ctx_right($ctx);
 
     } else {
-	use Data::Dumper;
 	unimp "Unrecognized context: ".Dumper($ctx);
     }
     $x->{ctx} = $ctx->copy;
@@ -583,7 +747,103 @@ sub P6C::ValueList::ctx_right {
 sub P6C::guard::ctx_right {
     my ($x, $ctx) = @_;
     $x->test->ctx_right(new P6C::Context type => 'bool');
-    $x->expr->ctx_right(new P6C::Context type => 'void');
+    $x->expr->ctx_right($ctx);
+    $x->{ctx} = $ctx->copy;
+}
+
+##############################
+sub P6C::loop::ctx_right {
+    my ($x, $ctx) = @_;
+    my $voidctx = $ctx->copy;
+    $voidctx->type('void');
+    # Remember that these can all be missing.
+    $x->init->ctx_right($voidctx) if $x->init;
+    $x->test->ctx_right(new P6C::Context type => 'bool') if $x->test;
+    $x->incr->ctx_right($voidctx) if $x->incr;
+    for (@{$x->block}) {
+	$_->ctx_right($voidctx);
+    }
+    $x->{ctx} = $ctx->copy;
+}
+
+##############################
+sub P6C::label::ctx_right { }
+sub P6C::debug_info::ctx_right { }
+
+##############################
+sub P6C::rule::ctx_right {
+    my ($x, $ctx) = @_;
+    $x->pat->ctx_right($ctx);
+    # XXX: need to do context for mod, probably process it as well,
+    # but that's non-trivial.
+    $x->{ctx} = $ctx;
+}
+
+sub P6C::rx_alt::ctx_right {
+    my ($x, $ctx) = @_;
+    my $rectx = new P6C::Context type => 'str';	# XXX: should be regex context?
+    $_->ctx_right($rectx) for @{$x->branches};
+    $x->{ctx} = $ctx;
+}
+
+sub P6C::rx_seq::ctx_right {
+    my ($x, $ctx) = @_;
+    for (@{$x->things}) {
+	$_->ctx_right($ctx);
+    }
+    $x->{ctx} = $ctx;
+}
+
+sub P6C::rx_hypo::ctx_right {
+    my ($x, $ctx) = @_;
+    $x->var->ctx_left($x->val);
+    $x->var->ctx_right($ctx);
+    $x->{ctx} = $ctx->copy;
+}
+
+sub P6C::rx_beg::ctx_right { }
+sub P6C::rx_end::ctx_right { }
+sub P6C::rx_cut::ctx_right { }
+sub P6C::rx_meta::ctx_right { }
+sub P6C::rx_oneof::ctx_right { }
+
+sub P6C::rx_atom::ctx_right {
+    my ($x, $ctx) = @_;
+
+    if (ref($x->atom) eq 'ARRAY') {
+	P6C::Context::block_ctx($x->atom, $ctx);
+
+    } elsif (UNIVERSAL::can($x->atom, 'type')
+	     && $x->atom->type eq 'PerlArray') {
+	# Arrays interpolate to an alternation of their elements, so
+	# they need to be in array context.
+	$x->atom->ctx_right(new P6C::Context type => 'PerlArray');
+	
+    } else {
+	$x->atom->ctx_right($ctx);
+    }
+    $x->{ctx} = $ctx;
+}
+
+sub P6C::rx_repeat::ctx_right {
+    my ($x, $ctx) = @_;
+    my $numctx = new P6C::Context type => 'int';
+    $x->min->ctx_right($numctx) if ref $x->min;
+    $x->max->ctx_right($numctx) if ref $x->max;
+    $x->thing->ctx_right($ctx);
+    $x->{ctx} = $ctx;
+}
+
+sub P6C::rx_assertion::ctx_right {
+    my ($x, $ctx) = @_;
+    my $strctx = new P6C::Context type => 'str';
+    $x->thing->ctx_right($strctx);
+}
+
+sub P6C::rx_call::ctx_right {
+    my ($x, $ctx) = @_;
+    $x->args->ctx_right(arg_context($x->name, $ctx));
+    $x->{ctx} = $ctx->copy;
 }
 
 1;
