@@ -20,7 +20,10 @@ the grammar rules in Parser.pm.
 =cut
 
 use strict;
+use Carp 'confess';
 use P6C::Nodes;
+use Data::Dumper;
+use P6C::Util qw(unimp flatten_leftop);
 
 ######################################################################
 
@@ -73,7 +76,6 @@ sub infix_left_seq {
     for (my $i = 1; $i < $#{$x}; $i += 2) {
 	my $op = ref($x[$i]) ? $x[$i]->tree : $x[$i];
 	unless (ref($x[$i + 1])) {
-	    use Data::Dumper;
 	    die Dumper(\@x);
 	}
 	my $r = $x[$i + 1]->tree;
@@ -86,10 +88,10 @@ sub infix_left_seq {
 # Define handlers for the simple right- and left-associative operators
 # here.
 BEGIN {
+    no strict 'refs';
     for (qw(apply match muldiv addsub bitshift bitand
 	    bitor logand logor log_AND log_OR comma semi
-	    _closure_args)) {
-	no strict 'refs';
+	    _closure_args rx_charclass)) {
 	*{'P6C::'.$_.'::tree'} = sub {
 	    my $x = shift;
 	    my $ret = infix_left_seq($x->[1]);
@@ -97,7 +99,6 @@ BEGIN {
 	};
     }
     for (qw(pow)) {
-	no strict 'refs';
 	*{'P6C::'.$_.'::tree'} = sub {
 	    my $x = shift;
 	    return infix_right_seq($x->[1]);
@@ -111,9 +112,11 @@ sub scalar_tree {
     shift->[1];
 }
 
-for (qw(name namepart sigil scope directive)) {
+{
     no strict 'refs';
-    *{"P6C\::$_\::tree"} = \&scalar_tree;
+    for (qw(name namepart sigil scope directive)) {
+	*{"P6C\::$_\::tree"} = \&scalar_tree;
+    }
 }
 
 ##############################
@@ -126,12 +129,14 @@ sub operator_tree {
     return $_;
 }
 
-for (qw(apply_op incr_op pow_op prefix_op filetest_op match_op
-	muldiv_op addsub_op bitshift_op compare_op bitand_op bitor_op
-	logand_op logor_op range_op assign_op comma_op semi_op
-	log_AND_op log_OR_op)) {
+{
     no strict 'refs';
-    *{"P6C\::$_\::tree"} = \&operator_tree;
+    for (qw(apply_op incr_op pow_op prefix_op filetest_op match_op
+	    muldiv_op addsub_op bitshift_op compare_op bitand_op bitor_op
+	    logand_op logor_op range_op assign_op comma_op semi_op
+	    log_AND_op log_OR_op)) {
+	*{"P6C\::$_\::tree"} = \&operator_tree;
+    }
 }
 
 ##############################
@@ -147,7 +152,7 @@ sub maybe_tree {
     } elsif (ref($x) =~ /^P6C::/) {
 	return $x->tree;
     } else {
-	die "maybe_tree: can't handle $t ($x)";
+	confess "maybe_tree: can't handle $t ($x)";
     }
 }
 
@@ -166,9 +171,11 @@ sub want_two_things {
 				       $x->[2]->tree];
 }
 
-for (qw(for given when while grep map sort)) {
+{
     no strict 'refs';
-    *{'P6C::want_for_'.$_.'::tree'} = \&want_two_things;
+    for (qw(for given when while grep map sort)) {
+	*{'P6C::want_for_'.$_.'::tree'} = \&want_two_things;
+    }
 }
 
 sub P6C::want_for_foreach::tree {
@@ -211,14 +218,15 @@ sub P6C::sv_literal::tree {
 	    $val = undef;
 	}
     } elsif (!ref($x->[1])) {
+	$x->[1] =~ s/_//g;
 	if ($x->[1] =~ /\./) {
-	    $type = 'PerlNum';
+	    $type = 'num';
 	} else {
-	    $type = 'PerlInt';
+	    $type = 'int';
 	}
 	$val = $x->[1];
     } else {
-	$type = 'PerlString';
+	$type = 'str';
 	$val = qq{"$x->[1][2]"};	# XXX: they're all just strings.
     }
     return new P6C::sv_literal type => $type, lval => $val;
@@ -264,15 +272,17 @@ sub P6C::variable::tree {
 			      implicit => $implicit);
 }
 
+sub P6C::scalar_var::tree {
+    my $x = shift;
+    my $sigil = $x->[1];
+    $x->[1] = ['sigil', $sigil];
+    return P6C::variable::tree($x);
+}
+
+*P6C::nonscalar_var::tree = *P6C::scalar_var::tree;
+
 ######################################################################
 # Operators
-# sub P6C::hype::tree {
-#     my $x = shift;
-#     if (@$x == 5) {
-# 	return new P6C::hype op => $x->[4]->tree;
-#     }
-#     return $x->[1]->tree;
-# }
 
 ##############################
 sub P6C::hv_indices::tree {
@@ -298,10 +308,8 @@ sub P6C::arglist::tree {
 sub P6C::subscript::tree {
     my $x = shift;
     if (@$x == 4) {
-	if (@{$x->[2]} > 0) {
-	    return $x->[2][0]->tree;
-	}
-	return undef;		# XXX: probably bad
+	# i.e. $x(23)
+	return new P6C::indices type => 'Sub', indices => maybe_tree($x->[2]);
     }
     my $type = ($x->[2] eq '{' ? 'PerlHash' : 'PerlArray');
     my $indices = $x->[5]->tree;
@@ -322,8 +330,11 @@ sub P6C::subscriptable::tree {
 	# XXX: fixme
 	return ['topical', maybe_tree($x->[4])];
     } else {
+	my $args = $x->[4]->tree;
+	$args = new P6C::ValueList
+	    vals => [$args ? flatten_leftop($args, ',') : ()];
 	return P6C::prefix->new(name => $x->[1]->tree,
-				args => $x->[4]->tree);
+				args => $args);
     }
 }
 
@@ -506,7 +517,7 @@ sub P6C::decl::tree {
 	return P6C::decl->new(vars => P6C::Tree::flatten_list($x->[3]),
 			      props => maybe_tree($x->[5]));
     }
-    return P6C::decl->new(vars => [$x->[1]->tree],
+    return P6C::decl->new(vars => $x->[1]->tree,
 			  props => maybe_tree($x->[2]));
 }
 
@@ -636,9 +647,9 @@ sub P6C::stmts::tree {
 	return [];
     }
     my @ret = ();
-    for (my $i = 0; $i <= $#{$x->[1]}; $i += 2) {
-	my $child = $x->[1][$i]->tree;
-	push @ret, $child if ($child);   # Ignore empty statements
+    foreach my $c (@{$x->[1]}) {
+        my @child = $c->tree;	# stmt return array when -g
+	push @ret, @child if (@child);   # Ignore empty statements
     }
     return [@ret];
 }
@@ -662,6 +673,16 @@ sub stmt_guard {
     return ($x->[1], $x->[3]->tree);
 }
 
+##############################
+# Debugging
+sub P6C::debug_info::tree {
+    my $x = shift;
+    my ($file, $l, $c, $stmt, $txt) = @$x;
+    my $di = bless [$file, $l, $c, $txt ], 'P6C::debug_info';
+    return ($di, $stmt->tree);
+}
+
+
 ##############################
 sub P6C::stmt::tree {
     my $x = shift;
@@ -697,7 +718,7 @@ sub P6C::stmt::tree {
 				   props => maybe_tree($x->[6]),
 				   block => $x->[7]->tree);
 
-    } elsif ($x->[2] eq 'sub') {
+    } elsif ($x->[2] =~ /^(?:sub|rule)$/) {
 	# Make sure we take care of declarations as well as
 	# definitions:
 	my $block = $x->[$#{$x}][1];
@@ -708,6 +729,7 @@ sub P6C::stmt::tree {
 				 name => $x->[4]->tree,
 				 props => maybe_tree($x->[7]),
 				 closure => $sc);
+	
     } elsif ($x->[2] eq ':') {	# label
 	return new P6C::label name => $x->[1]->tree;
 
@@ -733,7 +755,7 @@ sub P6C::closure::tree {
     my ($block, $arg_index);
     $block = $x->[$#{$x}]->tree;
     if (@$x == 2) {		# bare block
-	return new P6C::closure params => undef, block => $block;
+	return new P6C::closure params => undef, block => $block, bare => 1;
     } if (@$x == 5) {		# no parens
 	$arg_index = 3;
     } else {			# with parens
@@ -742,7 +764,9 @@ sub P6C::closure::tree {
     if (@{$x->[$arg_index]} == 0) {
 	return new P6C::closure params => undef, block => $block;
     }
+
     my $p = $x->[$arg_index][0]->tree;
+
     return new P6C::closure params => $p, block => $block;
 }
 
@@ -767,10 +791,15 @@ sub P6C::maybe_decl::tree {
 ##############################
 sub P6C::maybe_comma::tree {
     my $x = shift;
-    if (@$x == 3) {
-	return $x->[2]->tree;
+    if (@$x == 2 && ref $x->[1]) {
+	return $x->[1]->tree;
     }
-    return undef;
+    return new P6C::ValueList vals => [];
+}
+
+sub P6C::bare_arglist::tree {
+    my $x = shift;
+    return $x->[2]->tree;
 }
 
 ##############################
@@ -782,4 +811,246 @@ sub P6C::maybe_namepart::tree {
     return undef;
 }
 
+##############################
+sub P6C::nothing::tree {
+    return new P6C::ValueList vals => [];
+}
+
+*P6C::no_args::tree = *P6C::nothing::tree;
+
+##############################
+sub P6C::rule::tree {
+    my $x = shift;
+    return new P6C::rule mod => maybe_tree($x->[1]), pat => $x->[3]->tree;
+}
+
+sub P6C::pattern::tree {
+    my $x = shift;
+    if (@$x == 4) {
+	return new P6C::rule pat => $x->[2]->tree;
+    }
+    return new P6C::rule mod => maybe_tree($x->[3]), pat => $x->[5]->tree,
+	immediate => ($x->[1] eq 'm');
+}
+
+sub P6C::rx_alt::tree {
+    my $x = shift;
+    return new P6C::rx_alt branches => [map { $_->tree } @{$x->[1]} ];
+}
+
+sub P6C::rx_seq::tree {
+    my $x = shift;
+    return new P6C::rx_seq things => [map { $_->tree } @{$x->[1]} ];
+}
+
+sub P6C::rx_maybe_hypo::tree {
+    my $x = shift;
+    if (@$x == 2) {
+	if (ref $x->[1]) {
+	    # rx_element
+	    return $x->[1]->tree;
+	} elsif ($x->[1] eq '^') {
+	    return new P6C::rx_beg;
+	} elsif ($x->[1] eq '$') {
+	    return new P6C::rx_end;
+	} elsif ($x->[1] eq '^^' || $x->[1] eq '$$') {
+	    unimp 'BOL or EOL';
+	} else {
+	    # Cut
+	    return new P6C::rx_cut level => length($x->[1]);
+	}
+    } elsif (@$x == 5) {
+	# Scalar hypo
+	return new P6C::rx_hypo var => $x->[1]->tree,
+	    val => $x->[4]->tree;
+    } else {
+	# array hypo
+	return new P6C::rx_hypo var => $x->[1]->tree,
+	    val => new P6C::rx_atom(repeat => $x->[6]->tree,
+				    atom => $x->[4]->tree);
+    }
+}
+
+sub P6C::rx_element::tree {
+    my $x = shift;
+    my $ret;
+    if (@$x == 4) {
+	# codeblock
+	$ret = new P6C::rx_atom atom => $x->[3]->tree;
+    } else {
+	# atom
+	$ret = $x->[1]->tree;
+	if (@{$x->[2]}) {
+	    my $rep = $x->[2][0]->tree;
+	    $rep->thing($ret);
+	    $ret = $rep;
+	}
+    }
+    return $ret;
+}
+
+sub P6C::rx_atom::tree {
+    my $x = shift;
+    my ($atom, $capture);
+    if (@$x == 5) {
+	if ($x->[1] =~ /\[\(/) {
+	    # group, capturing or otherwise.
+	    $atom = $x->[3]->tree;
+	    $capture = $x->[1] eq '(';
+	} else {
+	    # modifiers
+	    return $x->[3]->tree;
+	}
+    } elsif (@$x == 6) {
+	# assertion
+	my $ret = $x->[4]->tree;
+	if (length($x->[3]) > 0) {
+	    $ret->negated(!$ret->negated);
+	}
+	return $ret;
+    } elsif (ref $x->[1]) {
+	# variable
+	$atom = $x->[1]->tree;
+    } elsif ($x->[1] eq '.') {
+	# metachar
+	$atom = new P6C::rx_meta name => '.';
+
+    } elsif ($x->[1] =~ /^\\(.+)/) {
+	# metachar
+	$atom = new P6C::rx_meta name => $1;
+    } else {
+	$atom = new P6C::sv_literal type => 'PerlString', lval => qq{"$x->[1]"};
+    }
+    return new P6C::rx_atom capture => $capture, atom => $atom;
+}
+
+sub P6C::rx_mod::tree {
+    my $x = shift;
+    my $ret = new P6C::rx_mod mod => $x->[2];
+    if (@{$x->[4]} > 0) {
+	$ret->args($x->[4][2]->tree);
+    }
+    return $ret;
+}
+
+sub repspec_item {
+    my $i = shift;
+    if (ref $i) {
+	return $i->tree;
+    } else {
+	return $i;
+    }
+}
+
+sub P6C::rx_repeat::tree {
+    my $x = shift;
+    my $greedy = length($x->[2]) == 0;
+    my ($min, $max);
+    if (@{$x->[1]} == 2) {
+	my $t = $x->[1][1];
+	if ($t eq '*') {
+	    $min = 0; $max = undef;
+	} elsif ($t eq '+') {
+	    $min = 1; $max = undef;
+	} else {
+	    # '?'
+	    $min = 0; $max = 1;	    
+	}
+    } else {
+	my $repspec = $x->[1][4];
+	if (@$repspec == 4) {
+	    # two items
+	    $min = repspec_item($repspec->[1][1]);
+	    $max = repspec_item($repspec->[3][1]);
+	} else {
+	    # one number
+	    $min = $max = 0 + $repspec->[1];
+	}
+    }
+    return new P6C::rx_repeat min => $min, max => $max, greedy => $greedy;
+}
+
+sub P6C::rx_assertion::tree {
+    my $x = shift;
+    if (@$x == 2) {
+	if (ref $x->[1]) {
+	    # Variable, subrule, or character class
+	    return $x->[1]->tree;
+	} elsif ($x->[1] eq '.') {
+	    # Single grapheme
+	    unimp "Logical grapheme";
+	} elsif ($x->[1] =~ /^'/) {
+	    return new P6C::rx_assertion
+		thing => P6C::sv_literal->new(type => 'str', lval => $x->[1]);
+	} else {
+	    die "internal error";
+	}
+    } elsif ($x->[1] eq '-') {
+	# Negated assertion
+	my $thing = $x->[3]->tree;
+	$thing->negated(!$thing->negated);
+	return $thing;
+    } elsif ($x->[1] eq '(') {
+	return new P6C::rx_assertion thing => $x->[2]->tree;
+    } elsif ($x->[1] eq '{') {
+	unimp "runtime-interpolated pattern";
+    } else {
+	die;
+    }
+}
+
+sub P6C::rx_rulename::tree {
+    my $x = shift;
+    return $x->[1]->tree;
+}
+
+sub P6C::rx_call::tree {
+    my $x = shift;
+    my $pat = $x->[1]->tree;
+    if ($x->[2] eq '(') {
+	# Call with args
+	return new P6C::rx_call name => $pat, args => $x->[4]->tree;
+    } elsif ($x->[2] eq ':') {
+	# Call with string
+	my $strval = $x->[4];
+	$strval =~ s/^\s*//;
+	return new P6C::rx_call name => $pat,
+	    args => new P6C::sv_literal type => 'str', lval => $strval;
+    } else {
+	# Call with pattern or no args
+	my $args = maybe_tree($x->[2]);
+	if (!defined($args) ||
+	    (@{$args->branches} == 1 && @{$args->branches(0)->things} == 0)) {
+	    # empty pattern => no args.
+	    $args = new P6C::ValueList vals => [];
+	}
+	return new P6C::rx_call name => $pat, args => $args;
+    }
+}
+
+sub P6C::rx_cc_neg::tree {
+    my $x = shift;
+    my $neg = $x->[1] eq '-';
+    my $class = $x->[2];
+    if (@$class == 4) {
+	# Bracketed assertion
+	my $ret = $class->[2]->tree;
+	$ret->negated(!$ret->negated) if $neg;
+	return $ret;
+
+    } elsif (ref $class->[1]) {
+	# Non-bracketed assertion
+	return new P6C::rx_assertion
+	    thing => new P6C::rx_call(name => $class->[1]->tree,
+				      args => new P6C::ValueList(vals => [])),
+	    negated => $neg;
+
+    } else {
+	# enumerated class
+	return new P6C::rx_oneof
+	    rep => substr($class->[1], 1, length($class->[1]) - 2),
+	    negated => $neg;
+    }
+}
+
 1;
