@@ -7,7 +7,7 @@
 ** This program is free software. It is subject to the same
 ** license as Parrot itself.
 **
-** $Id: packfile.c,v 1.52 2002/06/12 22:58:29 dmlloyd Exp $
+** $Id: packfile.c,v 1.55 2002/08/19 23:14:48 tom Exp $
 **
 ** History:
 **  Rework by Melvin; new bytecode format, make bytecode portable.
@@ -148,7 +148,8 @@ PackFile_fetch_nv(struct PackFile *pf, opcode_t *stream) {
     FLOATVAL f;
     if(pf->fetch_nv == NULL) {
 #if TRACE_PACKFILE
-        fprintf(stderr, "PackFile_fetch_nv: Native [%d bytes]..\n", sizeof(FLOATVAL));
+        fprintf(stderr, "PackFile_fetch_nv: Native [%d bytes]..\n", 
+                sizeof(FLOATVAL));
 #endif
         memcpy(&f, stream, sizeof(FLOATVAL));
         return f;
@@ -298,7 +299,7 @@ PackFile_unpack(struct Parrot_Interp *interpreter, struct PackFile *self,
      * Map the header on top of the buffer later when we are sure
      * we have alignment done right.
      */
-    cursor = (opcode_t*)((char*)packed + PACKFILE_HEADER_BYTES);
+    cursor = packed + PACKFILE_HEADER_BYTES/sizeof(opcode_t);
     memcpy(header, packed, PACKFILE_HEADER_BYTES);
     if(header->wordsize != sizeof(opcode_t)) {
         self->need_wordsize = 1;
@@ -320,9 +321,10 @@ PackFile_unpack(struct Parrot_Interp *interpreter, struct PackFile *self,
      * FIXME
      */
     if(self->need_wordsize) {
-        fprintf(stderr, "PackFile_unpack: Unimplemented wordsize transform.\n");
-        fprintf(stderr, "File has wordsize: %d (native is %dn", header->wordsize,
-                        sizeof(opcode_t));
+        fprintf(stderr, 
+                "PackFile_unpack: Unimplemented wordsize transform.\n");
+        fprintf(stderr, "File has wordsize: %d (native is %d)\n", 
+                header->wordsize,  sizeof(opcode_t));
         return 0;    
     }    
  
@@ -359,13 +361,15 @@ PackFile_unpack(struct Parrot_Interp *interpreter, struct PackFile *self,
         return 0;
     }
 
-    if (!PackFile_FixupTable_unpack(self->fixup_table, cursor, header->fixup_ss)) {
+    if (!PackFile_FixupTable_unpack(self->fixup_table, cursor, 
+                                    header->fixup_ss)) {
         fprintf(stderr,
                 "PackFile_unpack: Error reading fixup table segment!\n");
         return 0;
     }
 
-    cursor += header->fixup_ss / sizeof(opcode_t);  /* Segment size is in bytes */
+    /* Segment size is in bytes */
+    cursor += header->fixup_ss / sizeof(opcode_t); 
 
     /*
      * Unpack the Constant Table Segment:
@@ -384,7 +388,8 @@ PackFile_unpack(struct Parrot_Interp *interpreter, struct PackFile *self,
         return 0;
     }
 
-    cursor += header->const_ss / sizeof(opcode_t);  /* Segment size is in bytes */
+    /* Segment size is in bytes */
+    cursor += header->const_ss / sizeof(opcode_t); 
 
     /*
      * Unpack the Byte Code Segment:
@@ -417,7 +422,8 @@ PackFile_unpack(struct Parrot_Interp *interpreter, struct PackFile *self,
             for(i = 0; i < (int)(self->byte_code_size / sizeof(opcode_t)); i++) {
                 self->byte_code[i] = PackFile_fetch_op(self, cursor++);
 #if TRACE_PACKFILE
-                fprintf(stderr, "op[%u]->[%u]\n", *(cursor-1), self->byte_code[i]);
+                fprintf(stderr, "op[%u]->[%u]\n", *(cursor-1),
+                        self->byte_code[i]);
 #endif
             }    
         }
@@ -657,6 +663,10 @@ PackFile_Constant_destroy(struct PackFile_Constant *self)
         }
         break;
 
+    case PFC_KEY:
+        self->key = NULL;
+        break;
+
     default:
         fprintf(stderr,
                 "PackFile_Constant_clear: Unrecognized type '%c' (%ld)!\n",
@@ -674,8 +684,8 @@ PackFile_Constant_destroy(struct PackFile_Constant *self)
 
 =item pack_size
 
-Determine the size of the buffer needed in order to pack the PackFile Constant into a
-contiguous region of memory.
+Determine the size of the buffer needed in order to pack the PackFile 
+Constant into a contiguous region of memory.
 
 =cut
 
@@ -686,6 +696,7 @@ PackFile_Constant_pack_size(struct PackFile_Constant *self)
 {
     opcode_t packed_size;
     opcode_t padded_size;
+    PMC *component;
 
     if (!self) {
         /* TODO: OK to gloss over this? */
@@ -712,6 +723,14 @@ PackFile_Constant_pack_size(struct PackFile_Constant *self)
         packed_size = 4 * sizeof(opcode_t) + padded_size;
         break;
 
+    case PFC_KEY:
+        packed_size = sizeof(opcode_t);
+
+        for (component = self->key; component; component = component->data) {
+            packed_size += 2 * sizeof(opcode_t);
+        }
+        break;
+
     default:
         packed_size = 0;
         break;
@@ -779,7 +798,12 @@ PackFile_Constant_unpack(struct Parrot_Interp *interpreter,
         break;
 
     case PFC_STRING:
-        rc = PackFile_Constant_unpack_string(interpreter, pf, self, cursor, size);
+        rc = PackFile_Constant_unpack_string(interpreter, pf, self, 
+                                             cursor, size);
+        break;
+
+    case PFC_KEY:
+        rc = PackFile_Constant_unpack_key(interpreter, pf, self, cursor, size);
         break;
 
     default:
@@ -808,7 +832,8 @@ Returns one (1) if everything is OK, else zero (0).
 ***************************************/
 
 INTVAL
-PackFile_Constant_unpack_number(struct PackFile * pf, struct PackFile_Constant *self,
+PackFile_Constant_unpack_number(struct PackFile * pf, 
+                                struct PackFile_Constant *self,
                                 opcode_t *packed, opcode_t packed_size)
 {
     opcode_t *cursor;
@@ -835,7 +860,8 @@ PackFile_Constant_unpack_number(struct PackFile * pf, struct PackFile_Constant *
      * determined by Configure.
      */
 #if TRACE_PACKFILE
-    fprintf(stderr, "FIXME: PackFile_Constant_unpack_number: assuming size of FLOATVAL!\n");
+    fprintf(stderr,
+            "FIXME: PackFile_Constant_unpack_number: assuming size of FLOATVAL!\n");
 #endif
     self->number = PackFile_fetch_nv(pf, (opcode_t *)cursor);
      
@@ -890,7 +916,9 @@ PackFile_Constant_unpack_string(struct Parrot_Interp *interpreter,
     flags = (UINTVAL)PackFile_fetch_op(pf, cursor++);
     encoding = PackFile_fetch_op(pf, cursor++);
     type = PackFile_fetch_op(pf, cursor++);
-    size = (size_t)PackFile_fetch_op(pf, cursor++); /* These may need to be separate */
+
+    /* These may need to be separate */
+    size = (size_t)PackFile_fetch_op(pf, cursor++);
 
 #if TRACE_PACKFILE
     printf("PackFile_Constant_unpack_string(): flags are 0x%04x...\n", flags);
@@ -910,6 +938,97 @@ PackFile_Constant_unpack_string(struct Parrot_Interp *interpreter,
     return 1;
 }
 
+/***************************************
+
+=item unpack_key
+
+Unpack a PackFile Constant from a block of memory. The format consists
+of a sequence of key atomes, each with the following format:
+
+  opcode_t type
+  opcode_t value
+
+Returns one (1) if everything is OK, else zero (0).
+
+=cut
+
+***************************************/
+
+INTVAL
+PackFile_Constant_unpack_key(struct Parrot_Interp *interpreter,
+                             struct PackFile * pf,
+                             struct PackFile_Constant *self,
+                             opcode_t *packed, opcode_t packed_size)
+{
+    opcode_t *cursor;
+    INTVAL components;
+    PMC *head;
+    PMC *tail;
+
+    UNUSED(packed_size);
+
+    if (!self) 
+    {
+        return 0;
+    }
+    
+    cursor = packed;
+
+    components = *cursor++;
+
+    head = tail = NULL;
+
+    while (components-- > 0) {
+        if (tail) {
+            tail->data = pmc_new(interpreter, enum_class_Key);
+            tail = tail->data;
+        }
+        else {
+            head = tail = pmc_new(interpreter, enum_class_Key);
+        }
+
+        tail->flags |= PMC_constant_FLAG;
+        
+        switch (*cursor++) {
+        case PARROT_ARG_IC:
+            tail->flags |= KEY_integer_FLAG;
+            tail->cache.int_val = *cursor++;
+            break;
+        case PARROT_ARG_NC:
+            tail->flags |= KEY_number_FLAG;
+            tail->cache.num_val = pf->const_table->constants[*cursor++]->number;
+            break;
+        case PARROT_ARG_SC:
+            tail->flags |= KEY_string_FLAG;
+            tail->cache.string_val = pf->const_table->constants[*cursor++]->string;
+            break;
+        case PARROT_ARG_I:
+            tail->flags |= KEY_integer_FLAG|KEY_register_FLAG;
+            tail->cache.int_val = *cursor++;
+            break;
+        case PARROT_ARG_N:
+            tail->flags |= KEY_number_FLAG|KEY_register_FLAG;
+            tail->cache.int_val = *cursor++;
+            break;
+        case PARROT_ARG_S:
+            tail->flags |= KEY_string_FLAG|KEY_register_FLAG;
+            tail->cache.int_val = *cursor++;
+            break;
+        case PARROT_ARG_P:
+            tail->flags |= KEY_pmc_FLAG|KEY_register_FLAG;
+            tail->cache.int_val = *cursor++;
+            break;
+        default:
+            return 0;
+        }
+    }
+
+    self->type = PFC_KEY;
+    self->key = head;
+
+    return 1;
+}
+
 /*
 
 =back
