@@ -2,7 +2,7 @@
  * pdb.c
  *
  * CVS Info
- *    $Id: pdb.c,v 1.2 2002/06/07 01:31:57 josh Exp $
+ *    $Id: pdb.c,v 1.5 2002/08/26 14:15:45 grunblatt Exp $
  * Overview:
  *    The Parrot debugger
  * History:
@@ -35,10 +35,10 @@ main(int argc, char *argv[])
         return 1;
     }
 
-    Parrot_init(interpreter);
+    Parrot_init(interpreter, (void*) &interpreter);
 
-    if (argc != 2) {
-        fprintf(stderr, "Usage: pdb programfile\n");
+    if (argc < 2) {
+        fprintf(stderr, "Usage: pdb programfile [program-options]\n");
         exit(1);
     }
 
@@ -54,7 +54,7 @@ main(int argc, char *argv[])
 
     Parrot_loadbc(interpreter, pf);
     PDB_printwelcome();
-    Parrot_debug(interpreter);
+    Parrot_debug(interpreter, argc-1, argv+1);
     Parrot_destroy(interpreter);
 
     return 0;
@@ -63,7 +63,7 @@ main(int argc, char *argv[])
 void
 PDB_printwelcome()
 {
-    fprintf(stderr, "Parrot Debugger 0.0.1\n");
+    fprintf(stderr, "Parrot Debugger 0.0.2\n");
 }
 
 /*
