@@ -1,7 +1,7 @@
 /* rxstacks.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: rxstacks.c,v 1.7 2002/07/04 18:50:42 mrjoltcola Exp $
+ *     $Id: rxstacks.c,v 1.8 2002/08/22 20:08:24 dan Exp $
  *  Overview:
  *     Regex stack handling routines for Parrot
  *  Data Structure and Algorithms:
@@ -101,6 +101,18 @@ intstack_pop(struct Parrot_Interp *interpreter, IntStack stack)
     return entry->value;
 }
 
+
+void intstack_free (struct Parrot_Interp *interpreter, IntStack stack)
+{
+    IntStack chunk, temp;
+
+    for (chunk = stack->next; chunk != stack; chunk = temp) {
+        temp = chunk->next;        
+        mem_sys_free(chunk);
+    }
+
+    mem_sys_free(stack);
+}   
 /*
  * Local variables:
  * c-indentation-style: bsd
